@@ -170,6 +170,7 @@ RAW_DATA_TYPES: List[InputFileType] = [
 class OpenAPIScope(Enum):
     Schemas = 'schemas'
     Paths = 'paths'
+    Tags = 'tags'
 
 
 class Error(Exception):
@@ -214,12 +215,14 @@ def generate(
     aliases: Optional[Mapping[str, str]] = None,
     disable_timestamp: bool = False,
     allow_population_by_field_name: bool = False,
+    allow_extra_fields: bool = False,
     apply_default_values_for_required_fields: bool = False,
     force_optional_for_required_fields: bool = False,
     class_name: Optional[str] = None,
     use_standard_collections: bool = False,
     use_schema_description: bool = False,
     use_field_description: bool = False,
+    use_default_kwarg: bool = False,
     reuse_model: bool = False,
     encoding: str = 'utf-8',
     enum_field_as_literal: Optional[LiteralType] = None,
@@ -244,6 +247,10 @@ def generate(
     original_field_name_delimiter: Optional[str] = None,
     use_double_quotes: bool = False,
     use_union_operator: bool = False,
+    collapse_root_models: bool = False,
+    special_field_name_prefix: Optional[str] = None,
+    remove_special_field_name_prefix: bool = False,
+    capitalise_enum_members: bool = False,
 ) -> None:
     remote_text_cache: DefaultPutDict[str, str] = DefaultPutDict()
     if isinstance(input_, str):
@@ -336,6 +343,7 @@ def generate(
         strip_default_none=strip_default_none,
         aliases=aliases,
         allow_population_by_field_name=allow_population_by_field_name,
+        allow_extra_fields=allow_extra_fields,
         apply_default_values_for_required_fields=apply_default_values_for_required_fields,
         force_optional_for_required_fields=force_optional_for_required_fields,
         class_name=class_name,
@@ -345,6 +353,7 @@ def generate(
         else None,
         use_schema_description=use_schema_description,
         use_field_description=use_field_description,
+        use_default_kwarg=use_default_kwarg,
         reuse_model=reuse_model,
         enum_field_as_literal=enum_field_as_literal,
         set_default_enum_member=set_default_enum_member,
@@ -368,6 +377,10 @@ def generate(
         original_field_name_delimiter=original_field_name_delimiter,
         use_double_quotes=use_double_quotes,
         use_union_operator=use_union_operator,
+        collapse_root_models=collapse_root_models,
+        special_field_name_prefix=special_field_name_prefix,
+        remove_special_field_name_prefix=remove_special_field_name_prefix,
+        capitalise_enum_members=capitalise_enum_members,
         **kwargs,
     )
 
