@@ -18,10 +18,10 @@ from datamodel_code_generator.types import chain_as_tuple
 
 
 class Constraints(ConstraintsBase):
-    gt: Optional[Union[int, float]] = Field(None, alias='exclusiveMinimum')
-    ge: Optional[Union[int, float]] = Field(None, alias='minimum')
-    lt: Optional[Union[int, float]] = Field(None, alias='exclusiveMaximum')
-    le: Optional[Union[int, float]] = Field(None, alias='maximum')
+    gt: Optional[Union[float, int]] = Field(None, alias='exclusiveMinimum')
+    ge: Optional[Union[float, int]] = Field(None, alias='minimum')
+    lt: Optional[Union[float, int]] = Field(None, alias='exclusiveMaximum')
+    le: Optional[Union[float, int]] = Field(None, alias='maximum')
     multiple_of: Optional[float] = Field(None, alias='multipleOf')
     min_items: Optional[int] = Field(None, alias='minItems')
     max_items: Optional[int] = Field(None, alias='maxItems')
@@ -69,6 +69,11 @@ class DataModelField(DataModelFieldBase):
     def field(self) -> Optional[str]:
         """for backwards compatibility"""
         result = str(self)
+        if self.use_default_kwarg and not result.startswith("Field(..."):
+            # Use `default=` for fields that have a default value so that type
+            # checkers using @dataclass_transform can infer the field as
+            # optional in __init__.
+            result = result.replace("Field(", "Field(default=")
         if result == "":
             return None
 
@@ -113,6 +118,9 @@ class DataModelField(DataModelFieldBase):
         if self.use_field_description:
             data.pop('description', None)  # Description is part of field docstring
 
+        if self.const:
+            data['const'] = True
+
         field_arguments = sorted(
             f"{k}={repr(v)}" for k, v in data.items() if v is not None
         )
@@ -152,6 +160,7 @@ class BaseModel(DataModel):
         path: Optional[Path] = None,
         description: Optional[str] = None,
         default: Any = UNDEFINED,
+        nullable: bool = False,
     ):
 
         methods: List[str] = [field.method for field in fields if field.method]
@@ -168,14 +177,18 @@ class BaseModel(DataModel):
             path=path,
             description=description,
             default=default,
+            nullable=nullable,
         )
 
         config_parameters: Dict[str, Any] = {}
 
         additionalProperties = self.extra_template_data.get('additionalProperties')
-        if additionalProperties is not None:
+        allow_extra_fields = self.extra_template_data.get('allow_extra_fields')
+        if additionalProperties is not None or allow_extra_fields:
             config_parameters['extra'] = (
-                'Extra.allow' if additionalProperties else 'Extra.forbid'
+                'Extra.allow'
+                if additionalProperties or allow_extra_fields
+                else 'Extra.forbid'
             )
             self._additional_imports.append(IMPORT_EXTRA)
 
@@ -189,6 +202,10 @@ class BaseModel(DataModel):
                 config_parameters['arbitrary_types_allowed'] = True
                 break
 
+        if isinstance(self.extra_template_data.get('config'), dict):
+            for key, value in self.extra_template_data['config'].items():
+                config_parameters[key] = value
+
         if config_parameters:
             from datamodel_code_generator.model.pydantic import Config
 
