@@ -1,4 +1,5 @@
 import re
+import sys
 from abc import ABC, abstractmethod
 from collections import OrderedDict, defaultdict
 from itertools import groupby
@@ -12,6 +13,7 @@ from typing import (
     Iterator,
     List,
     Mapping,
+    NamedTuple,
     Optional,
     Sequence,
     Set,
@@ -31,6 +33,7 @@ from datamodel_code_generator.model.base import (
     ALL_MODEL,
     UNDEFINED,
     BaseClassDataType,
+    ConstraintsBase,
     DataModel,
     DataModelFieldBase,
 )
@@ -84,7 +87,7 @@ def dump_templates(templates: List[DataModel]) -> str:
 ReferenceMapSet = Dict[str, Set[str]]
 SortedDataModels = Dict[str, DataModel]
 
-MAX_RECURSION_COUNT: int = 100
+MAX_RECURSION_COUNT: int = sys.getrecursionlimit()
 
 
 def sort_data_models(
@@ -97,6 +100,7 @@ def sort_data_models(
         sorted_data_models = OrderedDict()
     if require_update_action_models is None:
         require_update_action_models = []
+    sorted_model_count: int = len(sorted_data_models)
 
     unresolved_references: List[DataModel] = []
     for model in unsorted_data_models:
@@ -116,7 +120,7 @@ def sort_data_models(
         else:
             unresolved_references.append(model)
     if unresolved_references:
-        if recursion_count:
+        if sorted_model_count != len(sorted_data_models) and recursion_count:
             try:
                 return sort_data_models(
                     unresolved_references,
@@ -124,7 +128,7 @@ def sort_data_models(
                     require_update_action_models,
                     recursion_count - 1,
                 )
-            except RecursionError:
+            except RecursionError:  # pragma: no cover
                 pass
 
         # sort on base_class dependency
@@ -141,7 +145,7 @@ def sort_data_models(
                 if indexes:
                     ordered_models.append(
                         (
-                            min(indexes),
+                            max(indexes),
                             model,
                         )
                     )
@@ -171,12 +175,10 @@ def sort_data_models(
             update_action_parent = set(require_update_action_models).intersection(
                 base_models
             )
-            if not unresolved_model and update_action_parent:
-                sorted_data_models[model.path] = model
-                require_update_action_models.append(model.path)
-                continue
             if not unresolved_model:
                 sorted_data_models[model.path] = model
+                if update_action_parent:
+                    require_update_action_models.append(model.path)
                 continue
             if not unresolved_model - unsorted_data_model_names:
                 sorted_data_models[model.path] = model
@@ -239,6 +241,50 @@ def title_to_class_name(title: str) -> str:
     return classname
 
 
+def _find_base_classes(model: DataModel) -> List[DataModel]:
+    return [
+        b.reference.source
+        for b in model.base_classes
+        if b.reference and isinstance(b.reference.source, DataModel)
+    ]
+
+
+def _find_field(
+    original_name: str, models: List[DataModel]
+) -> Optional[DataModelFieldBase]:
+    def _find_field_and_base_classes(
+        model_: DataModel,
+    ) -> Tuple[Optional[DataModelFieldBase], List[DataModel]]:
+        for field_ in model_.fields:
+            if field_.original_name == original_name:
+                return field_, []
+        return None, _find_base_classes(model_)  # pragma: no cover
+
+    for model in models:
+        field, base_models = _find_field_and_base_classes(model)
+        if field:
+            return field
+        models.extend(base_models)  # pragma: no cover
+
+    return None  # pragma: no cover
+
+
+def _copy_data_types(data_types: List[DataType]) -> List[DataType]:
+    copied_data_types: List[DataType] = []
+    for data_type_ in data_types:
+        if data_type_.reference:
+            copied_data_types.append(
+                data_type_.__class__(reference=data_type_.reference)
+            )
+        elif data_type_.data_types:
+            copied_data_type = data_type_.copy()
+            copied_data_type.data_types = _copy_data_types(data_type_.data_types)
+            copied_data_types.append(copied_data_type)
+        else:
+            copied_data_types.append(data_type_.copy())
+    return copied_data_types
+
+
 class Result(BaseModel):
     body: str
     source: Optional[Path]
@@ -277,12 +323,14 @@ class Parser(ABC):
         aliases: Optional[Mapping[str, str]] = None,
         allow_population_by_field_name: bool = False,
         apply_default_values_for_required_fields: bool = False,
+        allow_extra_fields: bool = False,
         force_optional_for_required_fields: bool = False,
         class_name: Optional[str] = None,
         use_standard_collections: bool = False,
         base_path: Optional[Path] = None,
         use_schema_description: bool = False,
         use_field_description: bool = False,
+        use_default_kwarg: bool = False,
         reuse_model: bool = False,
         encoding: str = 'utf-8',
         enum_field_as_literal: Optional[LiteralType] = None,
@@ -309,6 +357,11 @@ class Parser(ABC):
         original_field_name_delimiter: Optional[str] = None,
         use_double_quotes: bool = False,
         use_union_operator: bool = False,
+        allow_responses_without_content: bool = False,
+        collapse_root_models: bool = False,
+        special_field_name_prefix: Optional[str] = None,
+        remove_special_field_name_prefix: bool = False,
+        capitalise_enum_members: bool = False,
     ):
         self.data_type_manager: DataTypeManager = data_type_manager_type(
             python_version=target_python_version,
@@ -339,6 +392,7 @@ class Parser(ABC):
         )
         self.use_schema_description: bool = use_schema_description
         self.use_field_description: bool = use_field_description
+        self.use_default_kwarg: bool = use_default_kwarg
         self.reuse_model: bool = reuse_model
         self.encoding: str = encoding
         self.enum_field_as_literal: Optional[LiteralType] = enum_field_as_literal
@@ -378,6 +432,9 @@ class Parser(ABC):
         if allow_population_by_field_name:
             self.extra_template_data[ALL_MODEL]['allow_population_by_field_name'] = True
 
+        if allow_extra_fields:
+            self.extra_template_data[ALL_MODEL]['allow_extra_fields'] = True
+
         if enable_faux_immutability:
             self.extra_template_data[ALL_MODEL]['allow_mutation'] = False
 
@@ -390,6 +447,9 @@ class Parser(ABC):
             custom_class_name_generator=custom_class_name_generator,
             base_path=self.base_path,
             original_field_name_delimiter=original_field_name_delimiter,
+            special_field_name_prefix=special_field_name_prefix,
+            remove_special_field_name_prefix=remove_special_field_name_prefix,
+            capitalise_enum_members=capitalise_enum_members,
         )
         self.class_name: Optional[str] = class_name
         self.wrap_string_literal: Optional[bool] = wrap_string_literal
@@ -404,6 +464,9 @@ class Parser(ABC):
             use_non_positive_negative_number_constrained_types
         )
         self.use_double_quotes = use_double_quotes
+        self.allow_responses_without_content = allow_responses_without_content
+        self.collapse_root_models = collapse_root_models
+        self.capitalise_enum_members = capitalise_enum_members
 
     @property
     def iter_source(self) -> Iterator[Source]:
@@ -452,6 +515,358 @@ class Parser(ABC):
     def parse_raw(self) -> None:
         raise NotImplementedError
 
+    def __delete_duplicate_models(self, models: List[DataModel]) -> None:
+        for model in models[:]:
+            if isinstance(model, self.data_model_root_type):
+                root_data_type = model.fields[0].data_type
+
+                # backward compatible
+                # Remove duplicated root model
+                if (
+                    root_data_type.reference
+                    and not root_data_type.is_dict
+                    and not root_data_type.is_list
+                    and root_data_type.reference.source in models
+                    and root_data_type.reference.name
+                    == self.model_resolver.get_class_name(
+                        model.reference.original_name, unique=False
+                    ).name
+                ):
+                    # Replace referenced duplicate model to original model
+                    for child in model.reference.children[:]:
+                        child.replace_reference(root_data_type.reference)
+                    models.remove(model)
+                    for data_type in model.all_data_types:
+                        if data_type.reference:
+                            data_type.remove_reference()
+                    continue
+
+                #  Custom root model can't be inherited on restriction of Pydantic
+                for child in model.reference.children:
+                    # inheritance model
+                    if isinstance(child, DataModel):
+                        for base_class in child.base_classes[:]:
+                            if base_class.reference == model.reference:
+                                child.base_classes.remove(base_class)
+                        if not child.base_classes:
+                            child.set_base_class()
+
+    @classmethod
+    def __replace_duplicate_name_in_module(cls, models: List[DataModel]) -> None:
+        scoped_model_resolver = ModelResolver(
+            exclude_names={i.alias or i.import_ for m in models for i in m.imports},
+            duplicate_name_suffix='Model',
+        )
+
+        model_names: Dict[str, DataModel] = {}
+        for model in models:
+            class_name: str = model.class_name
+            generated_name: str = scoped_model_resolver.add(
+                model.path, class_name, unique=True, class_name=True
+            ).name
+            if class_name != generated_name:
+                model.class_name = generated_name
+            model_names[model.name] = model
+
+        for model in models:
+            duplicate_name = model.duplicate_class_name
+            # check only first desired name
+            if duplicate_name and duplicate_name not in model_names:
+                model.class_name = duplicate_name
+
+    @classmethod
+    def __change_from_import(
+        cls,
+        models: List[DataModel],
+        imports: Imports,
+        scoped_model_resolver: ModelResolver,
+        init: bool,
+    ) -> None:
+        for model in models:
+            imports.append(model.imports)
+            for data_type in model.all_data_types:
+                # To change from/import
+
+                if not data_type.reference or data_type.reference.source in models:
+                    # No need to import non-reference model.
+                    # Or, Referenced model is in the same file. we don't need to import the model
+                    continue
+
+                if isinstance(data_type, BaseClassDataType):
+                    from_ = ''.join(relative(model.module_name, data_type.full_name))
+                    import_ = data_type.reference.short_name
+                    full_path = from_, import_
+                else:
+                    from_, import_ = full_path = relative(
+                        model.module_name, data_type.full_name
+                    )
+
+                alias = scoped_model_resolver.add(full_path, import_).name
+
+                name = data_type.reference.short_name
+                if from_ and import_ and alias != name:
+                    data_type.alias = f'{alias}.{name}'
+
+                if init:
+                    from_ = "." + from_
+                imports.append(Import(from_=from_, import_=import_, alias=alias))
+
+    @classmethod
+    def __extract_inherited_enum(cls, models: List[DataModel]) -> None:
+        for model in models[:]:
+            if model.fields:
+                continue
+            enums: List[Enum] = []
+            for base_model in model.base_classes:
+                if not base_model.reference:
+                    continue
+                source_model = base_model.reference.source
+                if isinstance(source_model, Enum):
+                    enums.append(source_model)
+            if enums:
+                models.insert(
+                    models.index(model),
+                    enums[0].__class__(
+                        fields=[f for e in enums for f in e.fields],
+                        description=model.description,
+                        reference=model.reference,
+                    ),
+                )
+                models.remove(model)
+
+    @classmethod
+    def __set_reference_default_value_to_field(cls, models: List[DataModel]) -> None:
+        for model in models:
+            for model_field in model.fields:
+                if not model_field.data_type.reference or model_field.has_default:
+                    continue
+                if isinstance(
+                    model_field.data_type.reference.source, DataModel
+                ):  # pragma: no cover
+                    if model_field.data_type.reference.source.default != UNDEFINED:
+                        model_field.default = (
+                            model_field.data_type.reference.source.default
+                        )
+
+    def __reuse_model(
+        self, models: List[DataModel], require_update_action_models: List[str]
+    ) -> None:
+        if not self.reuse_model:
+            return None
+        model_cache: Dict[Tuple[str, ...], Reference] = {}
+        duplicates = []
+        for model in models[:]:
+            model_key = tuple(
+                to_hashable(v)
+                for v in (
+                    model.base_classes,
+                    model.extra_template_data,
+                    model.fields,
+                )
+            )
+            cached_model_reference = model_cache.get(model_key)
+            if cached_model_reference:
+                if isinstance(model, Enum):
+                    for child in model.reference.children[:]:
+                        # child is resolved data_type by reference
+                        data_model = get_most_of_parent(child)
+                        # TODO: replace reference in all modules
+                        if data_model in models:  # pragma: no cover
+                            child.replace_reference(cached_model_reference)
+                    duplicates.append(model)
+                else:
+                    index = models.index(model)
+                    inherited_model = model.__class__(
+                        fields=[],
+                        base_classes=[cached_model_reference],
+                        description=model.description,
+                        reference=Reference(
+                            name=model.name,
+                            path=model.reference.path + '/reuse',
+                        ),
+                    )
+                    if cached_model_reference.path in require_update_action_models:
+                        require_update_action_models.append(inherited_model.path)
+                    models.insert(index, inherited_model)
+                    models.remove(model)
+
+            else:
+                model_cache[model_key] = model.reference
+
+        for duplicate in duplicates:
+            models.remove(duplicate)
+
+    def __collapse_root_models(
+        self, models: List[DataModel], unused_models: List[DataModel]
+    ) -> None:
+        if not self.collapse_root_models:
+            return None
+        for model in models:
+            for model_field in model.fields:
+                for data_type in model_field.data_type.all_data_types:
+                    reference = data_type.reference
+                    if not reference or not isinstance(
+                        reference.source, self.data_model_root_type
+                    ):
+                        continue
+
+                    # Use root-type as model_field type
+                    root_type_model = reference.source
+                    root_type_field = root_type_model.fields[0]
+
+                    if (
+                        self.field_constraints
+                        and isinstance(root_type_field.constraints, ConstraintsBase)
+                        and root_type_field.constraints.has_constraints
+                        and any(
+                            d
+                            for d in model_field.data_type.all_data_types
+                            if d.is_dict or d.is_list or d.is_union
+                        )
+                    ):
+                        continue
+
+                    # set copied data_type
+                    copied_data_type = root_type_field.data_type.copy()
+                    if isinstance(data_type.parent, self.data_model_field_type):
+
+                        # for field
+                        # override empty field by root-type field
+                        model_field.extras = dict(
+                            root_type_field.extras, **model_field.extras
+                        )
+                        if self.field_constraints:
+                            if isinstance(
+                                root_type_field.constraints, ConstraintsBase
+                            ):  # pragma: no cover
+                                model_field.constraints = root_type_field.constraints.copy(
+                                    update={
+                                        k: v
+                                        for k, v in model_field.constraints.dict().items()
+                                        if v is not None
+                                    }
+                                    if isinstance(
+                                        model_field.constraints, ConstraintsBase
+                                    )
+                                    else {}
+                                )
+                        else:
+                            pass
+                            # skip function type-hint kwargs overriding
+
+                        data_type.parent.data_type = copied_data_type
+                    elif isinstance(data_type.parent, DataType):
+                        # for data_type
+                        data_type_id = id(data_type)
+                        data_type.parent.data_types = [
+                            d
+                            for d in (*data_type.parent.data_types, copied_data_type)
+                            if id(d) != data_type_id
+                        ]
+                    else:  # pragma: no cover
+                        continue
+
+                    data_type.remove_reference()
+
+                    root_type_model.reference.children = [
+                        c for c in root_type_model.reference.children if c.parent
+                    ]
+
+                    if not root_type_model.reference.children:
+                        unused_models.append(root_type_model)
+
+    def __set_default_enum_member(
+        self,
+        models: List[DataModel],
+        imports: Imports,
+        scoped_model_resolver: ModelResolver,
+        init: bool,
+    ) -> None:
+        if not self.set_default_enum_member:
+            return None
+        for model in models:
+            for model_field in model.fields:
+                if not model_field.default:
+                    continue
+                for data_type in model_field.data_type.all_data_types:
+                    if data_type.reference and isinstance(
+                        data_type.reference.source, Enum
+                    ):  # pragma: no cover
+                        enum_member = data_type.reference.source.find_member(
+                            model_field.default
+                        )
+                        if enum_member:
+                            model_field.default = enum_member
+                            enum_member_enum = enum_member.enum
+                            if enum_member_enum in models:
+                                continue
+                            scoped_model_resolver.get(enum_member_enum.path)
+                            if isinstance(enum_member_enum, Modular):
+                                enum_member_enum_name = f'{enum_member_enum.module_name}.{enum_member.enum.name}'
+                            else:
+                                enum_member_enum_name = enum_member.enum.name
+                            from_, import_ = full_path = relative(
+                                model.module_name, enum_member_enum_name
+                            )
+
+                            alias = scoped_model_resolver.add(full_path, import_).name
+
+                            name = data_type.reference.short_name
+                            if from_ and import_ and alias != name:
+                                enum_member.alias = f'{alias}.{name}'
+
+                            if init:
+                                from_ += "."
+                            imports.append(
+                                Import(from_=from_, import_=import_, alias=alias)
+                            )
+
+    def __override_required_field(
+        self,
+        models: List[DataModel],
+    ) -> None:
+        for model in models:
+            if isinstance(model, (Enum, self.data_model_root_type)):
+                continue
+            for index, model_field in enumerate(model.fields[:]):
+                data_type = model_field.data_type
+                if (
+                    not model_field.original_name
+                    or data_type.data_types
+                    or data_type.reference
+                    or data_type.type
+                    or data_type.literals
+                    or data_type.dict_key
+                ):
+                    continue
+
+                original_field = _find_field(
+                    model_field.original_name, _find_base_classes(model)
+                )
+                if not original_field:  # pragma: no cover
+                    model.fields.remove(model_field)
+                    continue
+                copied_original_field = original_field.copy()
+                if original_field.data_type.reference:
+                    data_type = self.data_type_manager.data_type(
+                        reference=original_field.data_type.reference,
+                    )
+                elif original_field.data_type.data_types:
+                    data_type = original_field.data_type.copy()
+                    data_type.data_types = _copy_data_types(
+                        original_field.data_type.data_types
+                    )
+                    for data_type_ in data_type.data_types:
+                        data_type_.parent = data_type
+                else:
+                    data_type = original_field.data_type.copy()
+                data_type.parent = copied_original_field
+                copied_original_field.data_type = data_type
+                copied_original_field.parent = model
+                copied_original_field.required = True
+                model.fields.insert(index, copied_original_field)
+                model.fields.remove(model_field)
+
     def parse(
         self,
         with_import: Optional[bool] = True,
@@ -480,7 +895,6 @@ class Parser(ABC):
         )
 
         results: Dict[Tuple[str, ...], Result] = {}
-
         module_key = lambda x: x.module_path
 
         # process in reverse order to correctly establish module levels
@@ -490,43 +904,16 @@ class Parser(ABC):
         )
 
         module_models: List[Tuple[Tuple[str, ...], List[DataModel]]] = []
+        unused_models: List[DataModel] = []
+        model_to_models: Dict[DataModel, List[DataModel]] = {}
 
         for module, models in (
             (k, [*v]) for k, v in grouped_models
         ):  # type: Tuple[str, ...], List[DataModel]
-
             for model in models:
-                if isinstance(model, self.data_model_root_type):
-                    root_data_type = model.fields[0].data_type
-
-                    # backward compatible
-                    # Remove duplicated root model
-                    if (
-                        root_data_type.reference
-                        and not root_data_type.is_dict
-                        and not root_data_type.is_list
-                        and root_data_type.reference.source in models
-                        and root_data_type.reference.name
-                        == self.model_resolver.get_class_name(
-                            model.reference.original_name, unique=False
-                        )
-                    ):
-                        # Replace referenced duplicate model to original model
-                        for child in model.reference.children[:]:
-                            child.replace_reference(root_data_type.reference)
-                        models.remove(model)
-                        continue
-
-                    #  Custom root model can't be inherited on restriction of Pydantic
-                    for child in model.reference.children:
-                        # inheritance model
-                        if isinstance(child, DataModel):
-                            for base_class in child.base_classes:
-                                if base_class.reference == model.reference:
-                                    child.base_classes.remove(base_class)
-                            if not child.base_classes:
-                                child.set_base_class()
-
+                model_to_models[model] = models
+            self.__delete_duplicate_models(models)
+            self.__replace_duplicate_name_in_module(models)
             module_models.append(
                 (
                     module,
@@ -534,24 +921,13 @@ class Parser(ABC):
                 )
             )
 
-            scoped_model_resolver = ModelResolver(
-                exclude_names={i.alias or i.import_ for m in models for i in m.imports},
-                duplicate_name_suffix='Model',
-            )
-
-            for model in models:
-                class_name: str = model.class_name
-                generated_name: str = scoped_model_resolver.add(
-                    model.path, class_name, unique=True, class_name=True
-                ).name
-                if class_name != generated_name:
-                    if '.' in model.reference.name:
-                        model.reference.name = (
-                            f"{model.reference.name.rsplit('.', 1)[0]}.{generated_name}"
-                        )
-                    else:
-                        model.reference.name = generated_name
+        class Processed(NamedTuple):
+            module: Tuple[str, ...]
+            models: List[DataModel]
+            init: bool
+            imports: Imports
 
+        processed_models: List[Processed] = []
         for module, models in module_models:
             init = False
             if module:
@@ -566,166 +942,24 @@ class Parser(ABC):
             else:
                 module = ('__init__.py',)
 
-            result: List[str] = []
             imports = Imports()
             scoped_model_resolver = ModelResolver()
+            self.__change_from_import(models, imports, scoped_model_resolver, init)
+            self.__extract_inherited_enum(models)
+            self.__set_reference_default_value_to_field(models)
+            self.__reuse_model(models, require_update_action_models)
+            self.__collapse_root_models(models, unused_models)
+            self.__set_default_enum_member(models, imports, scoped_model_resolver, init)
+            self.__override_required_field(models)
 
-            for model in models:
-                imports.append(model.imports)
-                for data_type in model.all_data_types:
-                    # To change from/import
+            processed_models.append(Processed(module, models, init, imports))
 
-                    if not data_type.reference or data_type.reference.source in models:
-                        # No need to import non-reference model.
-                        # Or, Referenced model is in the same file. we don't need to import the model
-                        continue
-
-                    if isinstance(data_type, BaseClassDataType):
-                        from_ = ''.join(
-                            relative(model.module_name, data_type.full_name)
-                        )
-                        import_ = data_type.reference.short_name
-                        full_path = from_, import_
-                    else:
-                        from_, import_ = full_path = relative(
-                            model.module_name, data_type.full_name
-                        )
-
-                    alias = scoped_model_resolver.add(full_path, import_).name
-
-                    name = data_type.reference.short_name
-                    if from_ and import_ and alias != name:
-                        data_type.alias = f'{alias}.{name}'
-
-                    if init:
-                        from_ = "." + from_
-                    imports.append(Import(from_=from_, import_=import_, alias=alias))
-
-            # extract inherited enum
-            for model in models:
-                if model.fields:
-                    continue
-                enums: List[Enum] = []
-                for base_model in model.base_classes:
-                    if not base_model.reference:
-                        continue
-                    source_model = base_model.reference.source
-                    if isinstance(source_model, Enum):
-                        enums.append(source_model)
-                if enums:
-                    models.insert(
-                        models.index(model),
-                        enums[0].__class__(
-                            fields=[f for e in enums for f in e.fields],
-                            description=model.description,
-                            reference=model.reference,
-                        ),
-                    )
-                    models.remove(model)
-
-            for model in models:
-                for model_field in model.fields:
-                    if not model_field.data_type.reference or model_field.has_default:
-                        continue
-                    if isinstance(
-                        model_field.data_type.reference.source, DataModel
-                    ):  # pragma: no cover
-                        if model_field.data_type.reference.source.default != UNDEFINED:
-                            model_field.default = (
-                                model_field.data_type.reference.source.default
-                            )
-
-            if self.reuse_model:
-                model_cache: Dict[Tuple[str, ...], Reference] = {}
-                duplicates = []
-                for model in models:
-                    model_key = tuple(
-                        to_hashable(v)
-                        for v in (
-                            model.base_classes,
-                            model.extra_template_data,
-                            model.fields,
-                        )
-                    )
-                    cached_model_reference = model_cache.get(model_key)
-                    if cached_model_reference:
-                        if isinstance(model, Enum):
-                            for child in model.reference.children[:]:
-                                # child is resolved data_type by reference
-                                data_model = get_most_of_parent(child)
-                                # TODO: replace reference in all modules
-                                if data_model in models:  # pragma: no cover
-                                    child.replace_reference(cached_model_reference)
-                            duplicates.append(model)
-                        else:
-                            index = models.index(model)
-                            inherited_model = model.__class__(
-                                fields=[],
-                                base_classes=[cached_model_reference],
-                                description=model.description,
-                                reference=Reference(
-                                    name=model.name,
-                                    path=model.reference.path + '/reuse',
-                                ),
-                            )
-                            if (
-                                cached_model_reference.path
-                                in require_update_action_models
-                            ):
-                                require_update_action_models.append(
-                                    inherited_model.path
-                                )
-                            models.insert(index, inherited_model)
-                            models.remove(model)
-
-                    else:
-                        model_cache[model_key] = model.reference
-
-                for duplicate in duplicates:
-                    models.remove(duplicate)
-
-            if self.set_default_enum_member:
-                for model in models:
-                    for model_field in model.fields:
-                        if not model_field.default:
-                            continue
-                        for data_type in model_field.data_type.all_data_types:
-                            if data_type.reference and isinstance(
-                                data_type.reference.source, Enum
-                            ):  # pragma: no cover
-                                enum_member = data_type.reference.source.find_member(
-                                    model_field.default
-                                )
-                                if enum_member:
-                                    model_field.default = enum_member
-                                    enum_member_enum = enum_member.enum
-                                    if enum_member_enum in models:
-                                        continue
-                                    scoped_model_resolver.get(enum_member_enum.path)
-                                    if isinstance(enum_member_enum, Modular):
-                                        enum_member_enum_name = f'{enum_member_enum.module_name}.{enum_member.enum.name}'
-                                    else:
-                                        enum_member_enum_name = enum_member.enum.name
-                                    from_, import_ = full_path = relative(
-                                        model.module_name, enum_member_enum_name
-                                    )
-
-                                    alias = scoped_model_resolver.add(
-                                        full_path, import_
-                                    ).name
-
-                                    name = data_type.reference.short_name
-                                    if from_ and import_ and alias != name:
-                                        enum_member.alias = f'{alias}.{name}'
-
-                                    if init:
-                                        from_ += "."
-                                    imports.append(
-                                        Import(
-                                            from_=from_, import_=import_, alias=alias
-                                        )
-                                    )
+        for unused_model in unused_models:
+            if unused_model in model_to_models[unused_model]:  # pragma: no cover
+                model_to_models[unused_model].remove(unused_model)
 
+        for module, models, init, imports in processed_models:
+            result: List[str] = []
             if with_import:
                 result += [str(self.imports), str(imports), '\n']
 
