@@ -50,12 +50,26 @@ from datamodel_code_generator.reference import ModelType, Reference, is_url
 from datamodel_code_generator.types import DataType, DataTypeManager, StrictTypes, Types
 
 
-def get_model_by_path(schema: Dict[str, Any], keys: List[str]) -> Dict[Any, Any]:
+def get_model_by_path(
+    schema: Union[Dict[str, Any], List[Any]], keys: Union[List[str], List[int]]
+) -> Dict[Any, Any]:
+    model: Union[Dict[Any, Any], List[Any]]
     if not keys:
-        return schema
+        model = schema
     elif len(keys) == 1:
-        return schema.get(keys[0], {})
-    return get_model_by_path(schema[keys[0]], keys[1:])
+        if isinstance(schema, dict):
+            model = schema.get(keys[0], {})  # type: ignore
+        else:  # pragma: no cover
+            model = schema[int(keys[0])]
+    elif isinstance(schema, dict):
+        model = get_model_by_path(schema[keys[0]], keys[1:])  # type: ignore
+    else:
+        model = get_model_by_path(schema[int(keys[0])], keys[1:])
+    if isinstance(model, dict):
+        return model
+    raise NotImplementedError(  # pragma: no cover
+        f'Does not support json pointer to array. schema={schema}, key={keys}'
+    )
 
 
 SPECIAL_PATH_FORMAT: str = '#-datamodel-code-generator-#-{}-#-special-#'
@@ -163,7 +177,7 @@ class JsonSchemaObject(BaseModel):
             return value.replace('#', '#/')
         return value
 
-    items: Union[List[JsonSchemaObject], JsonSchemaObject, None]
+    items: Union[List[JsonSchemaObject], JsonSchemaObject, bool, None]
     uniqueItems: Optional[bool]
     type: Union[str, List[str], None]
     format: Optional[str]
@@ -246,6 +260,10 @@ class JsonSchemaObject(BaseModel):
             return get_ref_type(self.ref)
         return None  # pragma: no cover
 
+    @cached_property
+    def type_has_null(self) -> bool:
+        return isinstance(self.type, list) and "null" in self.type
+
 
 @lru_cache()
 def get_ref_type(ref: str) -> JSONReference:
@@ -280,6 +298,7 @@ DEFAULT_FIELD_KEYS: Set[str] = {
     'description',
     'discriminator',
     'title',
+    'const',
 }
 
 EXCLUDE_FIELD_KEYS = (set(JsonSchemaObject.__fields__) - DEFAULT_FIELD_KEYS) | {
@@ -313,12 +332,14 @@ class JsonSchemaParser(Parser):
         aliases: Optional[Mapping[str, str]] = None,
         allow_population_by_field_name: bool = False,
         apply_default_values_for_required_fields: bool = False,
+        allow_extra_fields: bool = False,
         force_optional_for_required_fields: bool = False,
         class_name: Optional[str] = None,
         use_standard_collections: bool = False,
         base_path: Optional[Path] = None,
         use_schema_description: bool = False,
         use_field_description: bool = False,
+        use_default_kwarg: bool = False,
         reuse_model: bool = False,
         encoding: str = 'utf-8',
         enum_field_as_literal: Optional[LiteralType] = None,
@@ -343,6 +364,11 @@ class JsonSchemaParser(Parser):
         original_field_name_delimiter: Optional[str] = None,
         use_double_quotes: bool = False,
         use_union_operator: bool = False,
+        allow_responses_without_content: bool = False,
+        collapse_root_models: bool = False,
+        special_field_name_prefix: Optional[str] = None,
+        remove_special_field_name_prefix: bool = False,
+        capitalise_enum_members: bool = False,
     ):
         super().__init__(
             source=source,
@@ -361,6 +387,7 @@ class JsonSchemaParser(Parser):
             strip_default_none=strip_default_none,
             aliases=aliases,
             allow_population_by_field_name=allow_population_by_field_name,
+            allow_extra_fields=allow_extra_fields,
             apply_default_values_for_required_fields=apply_default_values_for_required_fields,
             force_optional_for_required_fields=force_optional_for_required_fields,
             class_name=class_name,
@@ -368,6 +395,7 @@ class JsonSchemaParser(Parser):
             base_path=base_path,
             use_schema_description=use_schema_description,
             use_field_description=use_field_description,
+            use_default_kwarg=use_default_kwarg,
             reuse_model=reuse_model,
             encoding=encoding,
             enum_field_as_literal=enum_field_as_literal,
@@ -392,6 +420,11 @@ class JsonSchemaParser(Parser):
             original_field_name_delimiter=original_field_name_delimiter,
             use_double_quotes=use_double_quotes,
             use_union_operator=use_union_operator,
+            allow_responses_without_content=allow_responses_without_content,
+            collapse_root_models=collapse_root_models,
+            special_field_name_prefix=special_field_name_prefix,
+            remove_special_field_name_prefix=remove_special_field_name_prefix,
+            capitalise_enum_members=capitalise_enum_members,
         )
 
         self.remote_object_cache: DefaultPutDict[str, Dict[str, Any]] = DefaultPutDict()
@@ -431,8 +464,54 @@ class JsonSchemaParser(Parser):
             self.enum_field_as_literal == LiteralType.One and len(obj.enum) == 1
         )
 
+    def get_object_field(
+        self,
+        *,
+        field_name: Optional[str],
+        field: JsonSchemaObject,
+        required: bool,
+        field_type: DataType,
+        alias: Optional[str],
+        original_field_name: Optional[str],
+    ) -> DataModelFieldBase:
+        if field.is_array or (
+            self.field_constraints
+            and not (
+                field.ref
+                or field.anyOf
+                or field.oneOf
+                or field.allOf
+                or field.is_object
+                or field.enum
+            )
+        ):
+            constraints: Optional[Mapping[str, Any]] = field.dict()
+        else:
+            constraints = None
+        return self.data_model_field_type(
+            name=field_name,
+            default=field.default,
+            data_type=field_type,
+            required=required,
+            alias=alias,
+            constraints=constraints,
+            nullable=field.nullable
+            if self.strict_nullable and (field.has_default or required)
+            else None,
+            strip_default_none=self.strip_default_none,
+            extras={**self.get_field_extras(field)},
+            use_annotated=self.use_annotated,
+            use_field_description=self.use_field_description,
+            use_default_kwarg=self.use_default_kwarg,
+            original_name=original_field_name,
+        )
+
     def get_data_type(self, obj: JsonSchemaObject) -> DataType:
         if obj.type is None:
+            if 'const' in obj.extras:
+                return self.data_type_manager.get_data_type_from_value(
+                    obj.extras['const']
+                )
             return self.data_type_manager.get_data_type(
                 Types.any,
             )
@@ -459,8 +538,7 @@ class JsonSchemaParser(Parser):
         return self.data_type(reference=reference)
 
     def set_additional_properties(self, name: str, obj: JsonSchemaObject) -> None:
-        if obj.additionalProperties is not None:
-            # TODO check additional property types.
+        if isinstance(obj.additionalProperties, bool):
             self.extra_template_data[name][
                 'additionalProperties'
             ] = obj.additionalProperties
@@ -472,47 +550,77 @@ class JsonSchemaParser(Parser):
     def parse_any_of(
         self, name: str, obj: JsonSchemaObject, path: List[str]
     ) -> List[DataType]:
-        return self.parse_list_item(name, obj.anyOf, path, obj)
+        data_types = self.parse_list_item(name, obj.anyOf, path, obj)
+        if not obj.properties and not obj.required:
+            return data_types
+        return [
+            self._parse_object_common_part(
+                name,
+                obj,
+                [*get_special_path('anyOfCommon', path), str(i)],
+                ignore_duplicate_model=True,
+                fields=[],
+                base_classes=[d.reference],
+                required=[],
+            )
+            for i, d in enumerate(data_types)
+            if d.reference
+        ]
 
     def parse_one_of(
         self, name: str, obj: JsonSchemaObject, path: List[str]
     ) -> List[DataType]:
-        return self.parse_list_item(name, obj.oneOf, path, obj)
+        data_types = self.parse_list_item(name, obj.oneOf, path, obj)
+        if not obj.properties and not obj.required:
+            return data_types
+        return [
+            self._parse_object_common_part(
+                name,
+                obj,
+                [*get_special_path('oneOfCommon', path), str(i)],
+                ignore_duplicate_model=True,
+                fields=[],
+                base_classes=[d.reference],
+                required=[],
+            )
+            for i, d in enumerate(data_types)
+            if d.reference
+        ]
 
-    def parse_all_of(
+    def _parse_object_common_part(
         self,
         name: str,
         obj: JsonSchemaObject,
         path: List[str],
-        ignore_duplicate_model: bool = False,
+        ignore_duplicate_model: bool,
+        fields: List[DataModelFieldBase],
+        base_classes: List[Reference],
+        required: List[str],
     ) -> DataType:
-        fields: List[DataModelFieldBase] = []
-        base_classes: List[Reference] = []
-        if len(obj.allOf) == 1 and not obj.properties:
-            single_obj = obj.allOf[0]
-            if single_obj.ref and single_obj.ref_type == JSONReference.LOCAL:
-                if get_model_by_path(self.raw_obj, single_obj.ref[2:].split('/')).get(
-                    'enum'
-                ):
-                    return self.get_ref_data_type(single_obj.ref)
-        for all_of_item in obj.allOf:
-            if all_of_item.ref:  # $ref
-                base_classes.append(self.model_resolver.add_ref(all_of_item.ref))
-            else:
-                fields.extend(
-                    self.parse_object_fields(
-                        all_of_item,
-                        path,
-                        get_module_name(name, None),
-                    )
-                )
+
         if obj.properties:
             fields.extend(
                 self.parse_object_fields(obj, path, get_module_name(name, None))
             )
         # ignore an undetected object
         if ignore_duplicate_model and not fields and len(base_classes) == 1:
+            self.model_resolver.delete(path)
             return self.data_type(reference=base_classes[0])
+        if required:
+            for field in fields:
+                if (field.original_name or field.name) in required:
+                    field.required = True
+        if obj.required:
+            field_name_to_field = {f.original_name or f.name: f for f in fields}
+            for required_ in obj.required:
+                if required_ in field_name_to_field:
+                    field_name_to_field[required_].required = True
+                else:
+                    fields.append(
+                        self.data_model_field_type(
+                            required=True, original_name=required_, data_type=DataType()
+                        )
+                    )
         if self.use_title_as_name and obj.title:
             name = obj.title
         reference = self.model_resolver.add(path, name, class_name=True, loaded=True)
@@ -531,6 +639,42 @@ class JsonSchemaParser(Parser):
 
         return self.data_type(reference=reference)
 
+    def parse_all_of(
+        self,
+        name: str,
+        obj: JsonSchemaObject,
+        path: List[str],
+        ignore_duplicate_model: bool = False,
+    ) -> DataType:
+        fields: List[DataModelFieldBase] = []
+        base_classes: List[Reference] = []
+        required: List[str] = []
+        if len(obj.allOf) == 1 and not obj.properties:
+            single_obj = obj.allOf[0]
+            if single_obj.ref and single_obj.ref_type == JSONReference.LOCAL:
+                if get_model_by_path(self.raw_obj, single_obj.ref[2:].split('/')).get(
+                    'enum'
+                ):
+                    return self.get_ref_data_type(single_obj.ref)
+        for all_of_item in obj.allOf:
+            if all_of_item.ref:  # $ref
+                base_classes.append(self.model_resolver.add_ref(all_of_item.ref))
+            else:
+                object_fields = self.parse_object_fields(
+                    all_of_item,
+                    path,
+                    get_module_name(name, None),
+                )
+                if object_fields:
+                    fields.extend(object_fields)
+                else:
+                    if all_of_item.required:
+                        required.extend(all_of_item.required)
+
+        return self._parse_object_common_part(
+            name, obj, path, ignore_duplicate_model, fields, base_classes, required
+        )
+
     def parse_object_fields(
         self, obj: JsonSchemaObject, path: List[str], module_name: Optional[str] = None
     ) -> List[DataModelFieldBase]:
@@ -563,25 +707,11 @@ class JsonSchemaParser(Parser):
                         strip_default_none=self.strip_default_none,
                         use_annotated=self.use_annotated,
                         use_field_description=self.use_field_description,
+                        original_name=original_field_name,
                     )
                 )
                 continue
 
-            if field.is_array or (
-                self.field_constraints
-                and not (
-                    field.ref
-                    or field.anyOf
-                    or field.oneOf
-                    or field.allOf
-                    or field.is_object
-                    or field.enum
-                )
-            ):
-                constraints: Optional[Mapping[str, Any]] = field.dict()
-            else:
-                constraints = None
-
             field_type = self.parse_item(modular_name, field, [*path, field_name])
 
             if self.force_optional_for_required_fields or (
@@ -591,20 +721,13 @@ class JsonSchemaParser(Parser):
             else:
                 required = original_field_name in requires
             fields.append(
-                self.data_model_field_type(
-                    name=field_name,
-                    default=field.default,
-                    data_type=field_type,
+                self.get_object_field(
+                    field_name=field_name,
+                    field=field,
                     required=required,
+                    field_type=field_type,
                     alias=alias,
-                    constraints=constraints,
-                    nullable=field.nullable
-                    if self.strict_nullable and (field.has_default or required)
-                    else None,
-                    strip_default_none=self.strip_default_none,
-                    extras={**self.get_field_extras(field)},
-                    use_annotated=self.use_annotated,
-                    use_field_description=self.use_field_description,
+                    original_field_name=original_field_name,
                 )
             )
         return fields
@@ -634,17 +757,42 @@ class JsonSchemaParser(Parser):
         )
         class_name = reference.name
         self.set_title(class_name, obj)
+        fields = self.parse_object_fields(obj, path, get_module_name(class_name, None))
+        if fields or not isinstance(obj.additionalProperties, JsonSchemaObject):
+            data_model_type_class = self.data_model_type
+        else:
+            fields.append(
+                self.get_object_field(
+                    field_name=None,
+                    field=obj.additionalProperties,
+                    required=False,
+                    original_field_name=None,
+                    field_type=self.data_type(
+                        data_types=[
+                            self.parse_item(
+                                # TODO: Improve naming for nested ClassName
+                                name,
+                                obj.additionalProperties,
+                                [*path, 'additionalProperties'],
+                            )
+                        ],
+                        is_dict=True,
+                    ),
+                    alias=None,
+                )
+            )
+            data_model_type_class = self.data_model_root_type
+
         self.set_additional_properties(class_name, obj)
-        data_model_type = self.data_model_type(
+        data_model_type = data_model_type_class(
             reference=reference,
-            fields=self.parse_object_fields(
-                obj, path, get_module_name(class_name, None)
-            ),
+            fields=fields,
             custom_base_class=self.base_class,
             custom_template_dir=self.custom_template_dir,
             extra_template_data=self.extra_template_data,
             path=self.current_source_path,
             description=obj.description if self.use_schema_description else None,
+            nullable=obj.type_has_null,
         )
         self.results.append(data_model_type)
         return self.data_type(reference=reference)
@@ -662,6 +810,7 @@ class JsonSchemaParser(Parser):
             singular_name = False
         if (
             parent
+            and not item.enum
             and item.has_constraint
             and (parent.has_constraint or self.field_constraints)
         ):
@@ -788,14 +937,18 @@ class JsonSchemaParser(Parser):
             else:
                 required = not obj.nullable and required
                 nullable = None
+        if isinstance(obj.items, JsonSchemaObject):
+            items: List[JsonSchemaObject] = [obj.items]
+        elif isinstance(obj.items, list):
+            items = obj.items
+        else:
+            items = []
 
         data_types: List[DataType] = [
             self.data_type(
                 data_types=self.parse_list_item(
                     name,
-                    [obj.items]
-                    if isinstance(obj.items, JsonSchemaObject)
-                    else obj.items or [],
+                    items,
                     path,
                     obj,
                     singular_name=singular_name,
@@ -827,6 +980,7 @@ class JsonSchemaParser(Parser):
             extras=self.get_field_extras(obj),
             use_annotated=self.use_annotated,
             use_field_description=self.use_field_description,
+            original_name=None,
         )
 
     def parse_array(
@@ -862,6 +1016,7 @@ class JsonSchemaParser(Parser):
                 extras=field.extras,
                 use_annotated=self.use_annotated,
                 use_field_description=self.use_field_description,
+                original_name=None,
             )
 
         data_model_root = self.data_model_root_type(
@@ -872,6 +1027,7 @@ class JsonSchemaParser(Parser):
             extra_template_data=self.extra_template_data,
             path=self.current_source_path,
             description=obj.description if self.use_schema_description else None,
+            nullable=obj.type_has_null,
         )
         self.results.append(data_model_root)
         return self.data_type(reference=reference)
@@ -888,23 +1044,17 @@ class JsonSchemaParser(Parser):
             data_type = self.data_type_manager.get_data_type_from_full_path(
                 obj.custom_type_path, is_custom_type=True
             )
-        elif obj.is_object or obj.anyOf or obj.oneOf:
-            data_types: List[DataType] = []
+        elif obj.anyOf or obj.oneOf:
             object_path = [*path, name]
-            if obj.is_object:
-                data_types.append(
-                    self.parse_object(
-                        name, obj, get_special_path('object', object_path)
-                    )
-                )
             if obj.anyOf:
-                data_types.extend(
-                    self.parse_any_of(name, obj, get_special_path('anyOf', object_path))
+                data_types: List[DataType] = self.parse_any_of(
+                    name, obj, get_special_path('anyOf', object_path)
                 )
-            if obj.oneOf:
-                data_types.extend(
-                    self.parse_one_of(name, obj, get_special_path('oneOf', object_path))
+            else:
+                data_types = self.parse_one_of(
+                    name, obj, get_special_path('oneOf', object_path)
                 )
+
             if len(data_types) > 1:
                 data_type = self.data_type(data_types=data_types)
             else:  # pragma: no cover
@@ -939,12 +1089,14 @@ class JsonSchemaParser(Parser):
                     extras=self.get_field_extras(obj),
                     use_annotated=self.use_annotated,
                     use_field_description=self.use_field_description,
+                    original_name=None,
                 )
             ],
             custom_base_class=self.base_class,
             custom_template_dir=self.custom_template_dir,
             extra_template_data=self.extra_template_data,
             path=self.current_source_path,
+            nullable=obj.type_has_null,
         )
         self.results.append(data_model_root_type)
         return self.data_type(reference=reference)
@@ -1012,6 +1164,7 @@ class JsonSchemaParser(Parser):
                     strip_default_none=self.strip_default_none,
                     has_default=obj.has_default,
                     use_field_description=self.use_field_description,
+                    original_name=None,
                 )
             )
 
@@ -1066,6 +1219,7 @@ class JsonSchemaParser(Parser):
                     use_annotated=self.use_annotated,
                     has_default=obj.has_default,
                     use_field_description=self.use_field_description,
+                    original_name=None,
                 )
             ],
             custom_base_class=self.base_class,
@@ -1073,6 +1227,7 @@ class JsonSchemaParser(Parser):
             extra_template_data=self.extra_template_data,
             path=self.current_source_path,
             default=obj.default if obj.has_default else UNDEFINED,
+            nullable=obj.type_has_null,
         )
         self.results.append(data_model_root_type)
         return self.data_type(reference=reference)
@@ -1143,8 +1298,9 @@ class JsonSchemaParser(Parser):
             if isinstance(obj.items, JsonSchemaObject):
                 self.parse_ref(obj.items, path)
             else:
-                for item in obj.items:
-                    self.parse_ref(item, path)
+                if isinstance(obj.items, list):
+                    for item in obj.items:
+                        self.parse_ref(item, path)
         if isinstance(obj.additionalProperties, JsonSchemaObject):
             self.parse_ref(obj.additionalProperties, path)
         if obj.patternProperties:
@@ -1168,8 +1324,9 @@ class JsonSchemaParser(Parser):
             if isinstance(obj.items, JsonSchemaObject):
                 self.parse_id(obj.items, path)
             else:
-                for item in obj.items:
-                    self.parse_id(item, path)
+                if isinstance(obj.items, list):
+                    for item in obj.items:
+                        self.parse_id(item, path)
         if isinstance(obj.additionalProperties, JsonSchemaObject):
             self.parse_id(obj.additionalProperties, path)
         if obj.patternProperties:
@@ -1212,6 +1369,8 @@ class JsonSchemaParser(Parser):
             self.parse_all_of(name, obj, path)
         elif obj.oneOf:
             self.parse_root_type(name, obj, path)
+        elif obj.anyOf:
+            self.parse_root_type(name, obj, path)
         elif obj.is_object:
             self.parse_object(name, obj, path)
         elif obj.enum:
@@ -1257,7 +1416,9 @@ class JsonSchemaParser(Parser):
 
         self._resolve_unparsed_json_pointer()
 
-    def _resolve_unparsed_json_pointer(self) -> None:
+    def _resolve_unparsed_json_pointer(
+        self, exclude_path_prefixes: Optional[List[str]] = None
+    ) -> None:
         model_count: int = len(self.results)
         for source in self.iter_source:
             path_parts = list(source.path.parts)
@@ -1271,6 +1432,14 @@ class JsonSchemaParser(Parser):
                 source.path.parent
             ), self.model_resolver.current_root_context(path_parts):
                 for reserved_ref in sorted(reserved_refs):
+                    if exclude_path_prefixes:
+                        reserved_ref_path_prefix = reserved_ref.split('#/', 1)[-1]
+                        if any(
+                            e
+                            for e in exclude_path_prefixes
+                            if reserved_ref_path_prefix.startswith(e)
+                        ):  # pragma: no cover
+                            continue
                     if self.model_resolver.add_ref(reserved_ref, resolved=True).loaded:
                         continue
                     # for root model
